# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Steffen Kockel s.kockel@groovecubes.de
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gettext
from gettext import gettext as _
gettext.textdomain('arduino-scope')

import gtk
import logging
logger = logging.getLogger('arduino_scope')

from arduino_scope_lib import Window
from arduino_scope_lib import preferences
from arduino_scope.AboutArduinoScopeDialog import AboutArduinoScopeDialog
from arduino_scope.PreferencesArduinoScopeDialog import PreferencesArduinoScopeDialog

from arduino_scope.helper import drawPath
from arduino_scope.helper import drawText
from arduino_scope.helper import getSerialConnection
from arduino_scope.helper import getSamplerate
from serial import Serial
import time
from os import mkdir
from os import getenv

# See arduino_scope_lib.Window.py for more details about how this class works
class ArduinoScopeWindow(Window):
    __gtype_name__ = "ArduinoScopeWindow"
    
    def finish_initializing(self, builder): # pylint: disable=E1002
        """Set up the main window
        """
        super(ArduinoScopeWindow, self).finish_initializing(builder)
        self.PreferencesDialog = PreferencesArduinoScopeDialog        
        self.AboutDialog = AboutArduinoScopeDialog
        self.SerialConnection = getSerialConnection()

        self.show_grid = self.builder.get_object("toggle_show_grid").active
        
        # TODO: make config options out of this
        self.graph_colors = ((0,0,1), (0,1,0), (1,0,0), 
                             (0,0,0.5), (0,0.5,0), (0.5,0,0))                             
        self.canvas = self.child.children()[1].children()[1]
        self.lastscanresults = []
        # this seems not to be best practice at all. But does the job. (sk)
        self.canvas = self.child.children()[1].children()[1]
        
    def prepare_canvas(self):
        # clear the canvas        
        self.canvas.window.clear()
        # setup the canvas % set background
        cr = self.canvas.window.cairo_create()
        ccolour = tuple([float(x) for x in preferences['canvas_color'].split(',')])
        gcolour =  tuple([float(x) for x in preferences['grid_color'].split(',')])
        cr.set_source_rgb(ccolour[0],ccolour[1],ccolour[2])
        size = self.canvas.window.get_size() 
        cr.rectangle(0,0,size[0]+40,size[1]+60)
        cr.fill()

        ms = getSamplerate(format="ms")
        width, height = size[0] , size[1] 

        # draw horizontal ruler
        drawPath(cr, (20,height-40), (width-20,height-40), color=gcolour, width=2)
        drawPath(cr, (20,20), (width-20,20), color=gcolour, width=2)
        ms_full_steps = (float(width)-40)/ ms
        off = 20
        for i in range(0, int(ms+1)):
            drawPath(cr, (off,height-40), (off,height-30), color=gcolour, width=1)
            drawText(cr,(off+3,height-25), str(i), color=gcolour)            
            drawPath(cr, (off,20), (off,10), color=gcolour, width=1)  
            if self.show_grid:
                drawPath(cr, (off, height-40), (off, 20), color=gcolour, width=1)    
            off += ms_full_steps           

        # draw vertical ruler        
        drawPath(cr, (width-20,0+20), (width-20,height-40), color=gcolour, width=2) 
        drawPath(cr, (20,20), (20,height-40), color=gcolour, width=2) 
        volt_full_steps = float(height-60)/5
        off = height-40
        for i in range(0, int(volt_full_steps)+1):
            drawPath(cr, (20,off), (10,off), color=gcolour, width=1)
            drawText(cr, (20-13,off-5), str(i), color=gcolour)
            drawPath(cr, (width-20,off), (width-10,off), color=gcolour, width=1)  
            if self.show_grid:
                drawPath(cr, (20,off), (width-20,off), color=gcolour, width=1)
            off -= volt_full_steps

        # the legend
        drawText(cr, (5, height-5), "V/ms", color=gcolour)

    def draw_graphs(self):
        if not hasattr(self, "lastscanresults") or len(self.lastscanresults) is 0:
            return
        
        size = self.canvas.window.get_size() 
        width, height = size[0], size[1]
        cr = self.canvas.window.cairo_create()
        for a in self.lastscanresults:
            port = int(a[0][1:])
            # should be 20, but looks better with set to 21
            x_off = x_old = 21
            y_off = y_old =  height-40
            x_steps = (float(width)-40) / 500
            y_steps = (float(height)-60) / 1024
            r,g,b = self.graph_colors[port]           

            for i in a[1][1:]:
                y = float(i.strip('\r\n')) * y_steps
                drawPath(cr, (x_old, y_old), (x_off, y_off-y), 
                         color=(r,g,b), width=2 )                    
                x_old = x_off                
                y_old = y_off-y
                x_off += x_steps

    def on_canvas_exposed(self,widget, event):
        self.prepare_canvas()
        # Draw the actual graph on the canvas 
        self.draw_graphs()

    def on_collectData_clicked(self, widget=None):
        self.lastscanresults = []
        aports = self.builder.get_object("channel_table").children()
        self.prepare_canvas()
        cr = self.canvas.window.cairo_create()
        # This also seems to be workaround. How to get the CheckButtons
        # names / ids? (sk)
        port = 5
        for a in aports:
            if not a.get_active():
                port -= 1
                continue

            logger.debug("port %s active" % port)
            # set the length of the bottom ruler to the ammount
            # of milliseconds needed to sample 500 samples. 
            #samplerate = float(500 / (int(preferences['samplerate']) / 1000))
            samplerate = getSamplerate(format="ms")
            
            # (re)Open and clean the serial connection
            self.SerialConnection.open()        
            self.SerialConnection.flushInput()        
            
            # Try to get 500 samples of the actual channel
            # in the given range of milliseconds from the 
            # arduino.
            while True:        
                self.SerialConnection.write('A%s\n' % port)
                time.sleep(0.3)
                data = self.SerialConnection.readlines()
                if len(data) != 501:
                    logger.debug("wrong data length: %s " % len(data))
                    continue
                
                cur_samplerate = float(data[0].strip('\r\n'))
                if  cur_samplerate != samplerate:
                    logger.debug("wrong samplerate! Got %sms, but need %sms" % (cur_samplerate, samplerate))
                    continue
                elif cur_samplerate == samplerate:
                    break

            data = tuple([a.strip('\r\n') for a in data])
            self.lastscanresults.append(['A%d' % port , data])
            # reset loop data and handle port number            
            data = None
            if port > 0:
                port -=1
        # draw the graphs
        self.SerialConnection.close()
        self.draw_graphs()

    def on_write_firmware_to_sketchbook_activate(self, widget):
        try:
            file = open("/usr/share/arduino-scope/firmware/ArduinoScopeFirmware.txt", "r").read()
            for line in [x for x in file.split("\n")]:
                if len(line) < 0:
                    continue
                if line.startswith("float firmwareVersion"):
                    version = float(line.split("=")[1].strip()[:-1])                               
                    name = "arduino_scope_%s" % str(version).replace(".","")
            path = "%s/sketchbook/%s" % (getenv("HOME"), name)
            mkdir(path) 
            sketch = open("%s/%s.ino" % (path,name), "w")
            sketch.writelines(file)
            sketch.close()
            logger.debug("Successfully added firmware to sketchbook")
        except StandardError,e:
            logger.debug(e)

    def on_show_grid_toggled(self, widget):
        self.show_grid = self.builder.get_object("toggle_show_grid").active       
        self.on_canvas_exposed(self, "foo")        
        logger.debug(self.show_grid)




        
